@@ -5,13 +5,15 @@ use super::{
         AggregatesCountStringMap, RecordsByLenMap, RecordsSensitivityByLen,
         ALL_SENSITIVITIES_INDEX,
     },
-    RecordsByStringKey, RecordsCountByStringKey,
+    AggregatedMetricByDataBlockValue, AggregatedMetricByString, AggregatesCountDataBlockValueMap,
+    RecordsByDataBlockValueKey, RecordsByStringKey, RecordsCountByStringKey,
 };
 use fnv::FnvHashMap;
 use itertools::Itertools;
 use log::info;
 use serde::{Deserialize, Serialize};
 use std::{
+    cmp::Ordering,
     fmt::Write as FmtWrite,
     io::{BufReader, BufWriter, Error, Write},
     sync::Arc,
@@ -113,6 +115,54 @@ impl AggregatedData {
             .collect()
     }
 
+    /// Calculates the records that contain rare combinations grouped by attribute.
+    /// This might contain duplicated records for different attribute names.
+    /// Unique combinations are also contained in this.
+    /// # Arguments:
+    /// * `resolution` - Reporting resolution used for data synthesis
+    pub fn calc_number_of_records_with_rare_combinations_per_attribute(
+        &self,
+        resolution: usize,
+    ) -> AggregatesCountDataBlockValueMap {
+        let mut rare_records_per_attribute = RecordsByDataBlockValueKey::default();
+
+        for (agg, count) in self.aggregates_count.iter() {
+            if count.count < resolution {
+                for value in agg.iter() {
+                    rare_records_per_attribute
+                        .entry(value.clone())
+                        .or_insert_with(RecordsSet::default)
+                        .extend(&count.contained_in_records);
+                }
+            }
+        }
+
+        rare_records_per_attribute
+            .drain()
+            .map(|(h, records)| (h, records.len()))
+            .collect()
+    }
+
+    /// Calculates the percentage of records that contain rare combinations grouped by attribute.
+    /// This might contain duplicated records for different attribute names.
+    /// Unique combinations are also contained in this.
+    /// # Arguments:
+    /// * `resolution` - Reporting resolution used for data synthesis
+    pub fn calc_percentage_of_records_with_rare_combinations_per_attribute(
+        &self,
+        resolution: usize,
+    ) -> AggregatedMetricByDataBlockValue {
+        let attr_counts = self.calc_single_attribute_counts();
+
+        self.calc_number_of_records_with_rare_combinations_per_attribute(resolution)
+            .drain()
+            .map(|(h, l)| {
+                let count = attr_counts[&h];
+                (h, 100.0 * l as f64 / count as f64)
+            })
+            .collect()
+    }
+
     #[inline]
     /// Check if the records len map contains a value across all lengths
     fn records_by_len_contains(records_by_len: &RecordsByLenMap, value: &usize) -> bool {
@@ -163,9 +213,11 @@ impl AggregatedData {
         writer: &mut T,
         aggregates_delimiter: char,
         combination_delimiter: &str,
+        case_insensitive_combinations_order: Option<bool>,
     ) -> Result<(), Error> {
         let n_records;
         let n_records_label;
+        let case_insensitive = case_insensitive_combinations_order.unwrap_or(false);
 
         if let Some(protected_number_of_records) = self.protected_number_of_records {
             n_records = protected_number_of_records;
@@ -181,13 +233,30 @@ impl AggregatedData {
         writer
             .write_all(format!("record_count{}{}\n", aggregates_delimiter, n_records).as_bytes())?;
 
-        for aggregate in self.aggregates_count.keys() {
+        for (aggregate, count) in self.aggregates_count.iter().sorted_by(|a, b| {
+            let len_cmp = a.0.len().cmp(&b.0.len());
+
+            if len_cmp == Ordering::Equal {
+                // same len, sort by count desc
+                b.1.count.cmp(&a.1.count)
+            } else {
+                // different len, sort by len asc
+                len_cmp
+            }
+        }) {
             writer.write_all(
                 format!(
                     "{}{}{}\n",
-                    aggregate.as_str_using_headers(&self.headers, combination_delimiter),
+                    if case_insensitive {
+                        aggregate.as_str_using_headers_case_insensitive_order(
+                            &self.headers,
+                            combination_delimiter,
+                        )
+                    } else {
+                        aggregate.as_str_using_headers(&self.headers, combination_delimiter)
+                    },
                     aggregates_delimiter,
-                    self.aggregates_count[aggregate].count
+                    count.count
                 )
                 .as_bytes(),
             )?
@@ -462,81 +531,45 @@ impl AggregatedData {
         rare_records.len()
     }
 
-    /// Calculates the records that contain rare combinations grouped by column name.
+    /// Calculates the percentage of records that contain rare combinations grouped by column name.
     /// This might contain duplicated records on different column names.
     /// Unique combinations are also contained in this.
     /// # Arguments:
     /// * `resolution` - Reporting resolution used for data synthesis
-    pub fn calc_records_with_rare_combinations_per_column(
+    pub fn calc_percentage_of_records_with_rare_combinations_per_column_str(
         &self,
         resolution: usize,
-    ) -> RecordsByStringKey {
-        let mut rare_records_per_column = RecordsByStringKey::default();
-
-        for (agg, count) in self.aggregates_count.iter() {
-            if count.count < resolution {
-                for value in agg.iter() {
-                    rare_records_per_column
-                        .entry(self.get_original_header_name(value.column_index))
-                        .or_insert_with(RecordsSet::default)
-                        .extend(&count.contained_in_records);
-                }
-            }
-        }
-        rare_records_per_column
-    }
-
-    /// Calculates the number of records that contain rare combinations grouped by column name.
-    /// This might contain duplicated records on different column names.
-    /// Unique combinations are also contained in this.
-    /// # Arguments:
-    /// * `resolution` - Reporting resolution used for data synthesis
-    pub fn calc_number_of_records_with_rare_combinations_per_column(
-        &self,
-        resolution: usize,
-    ) -> RecordsCountByStringKey {
-        self.calc_records_with_rare_combinations_per_column(resolution)
+    ) -> AggregatedMetricByString {
+        self.calc_percentage_of_records_with_rare_combinations_per_attribute(resolution)
             .drain()
-            .map(|(h, records)| (h, records.len()))
+            .map(|(value, percentage)| (value.column_index, percentage))
+            // group_by only works on consecutive elements
+            .sorted_by_key(|(column_index, _percentage)| *column_index)
+            .group_by(|(column_index, _percentage)| *column_index)
+            .into_iter()
+            .map(|(column_index, group)| {
+                let percentages = group.map(|(_, percentage)| percentage).collect_vec();
+
+                (
+                    self.get_original_header_name(column_index),
+                    percentages.iter().sum::<f64>() / (percentages.len() as f64),
+                )
+            })
             .collect()
     }
 
-    /// Calculates the records that contain rare combinations grouped by attribute.
+    /// Calculates the percentage of records that contain rare combinations grouped by attribute str.
     /// This might contain duplicated records for different attribute names.
     /// Unique combinations are also contained in this.
     /// # Arguments:
     /// * `resolution` - Reporting resolution used for data synthesis
-    pub fn calc_records_with_rare_combinations_per_attribute(
+    pub fn calc_percentage_of_records_with_rare_combinations_per_attribute_str(
         &self,
         resolution: usize,
-    ) -> RecordsByStringKey {
-        let mut rare_records_per_attribute = RecordsByStringKey::default();
-
-        for (agg, count) in self.aggregates_count.iter() {
-            if count.count < resolution {
-                for value in agg.iter() {
-                    rare_records_per_attribute
-                        .entry(self.get_original_attribute_as_str(value))
-                        .or_insert_with(RecordsSet::default)
-                        .extend(&count.contained_in_records);
-                }
-            }
-        }
-        rare_records_per_attribute
-    }
-
-    /// Calculates the number of records that contain rare combinations grouped by attribute.
-    /// This might contain duplicated records for different attribute names.
-    /// Unique combinations are also contained in this.
-    /// # Arguments:
-    /// * `resolution` - Reporting resolution used for data synthesis
-    pub fn calc_number_of_records_with_rare_combinations_per_attribute(
-        &self,
-        resolution: usize,
-    ) -> RecordsCountByStringKey {
-        self.calc_records_with_rare_combinations_per_attribute(resolution)
+    ) -> AggregatedMetricByString {
+        self.calc_percentage_of_records_with_rare_combinations_per_attribute(resolution)
             .drain()
-            .map(|(h, records)| (h, records.len()))
+            .map(|(h, p)| (self.get_original_attribute_as_str(&h), p))
             .collect()
     }
 
@@ -842,11 +875,14 @@ impl AggregatedData {
     /// * `aggregates_delimiter` - Delimiter to use when writing to `aggregates_path`
     /// * `combination_delimiter` - Delimiter used to join combinations and format then
     /// as strings
+    /// * `case_insensitive_combinations_order` - True if the attribute combinations ordering should be
+    /// sorted using case insensitive comparison
     pub fn write_aggregates_count(
         &self,
         aggregates_path: &str,
         aggregates_delimiter: char,
         combination_delimiter: &str,
+        case_insensitive_combinations_order: Option<bool>,
     ) -> Result<(), Error> {
         info!("writing file {}", aggregates_path);
 
@@ -856,6 +892,7 @@ impl AggregatedData {
             &mut std::io::BufWriter::new(std::fs::File::create(aggregates_path)?),
             aggregates_delimiter,
             combination_delimiter,
+            case_insensitive_combinations_order,
         )
     }
 
@@ -864,10 +901,13 @@ impl AggregatedData {
     /// * `aggregates_delimiter` - Delimiter to used for the CSV file
     /// * `combination_delimiter` - Delimiter used to join combinations and format then
     /// as strings
+    /// * `case_insensitive_combinations_order` - True if the attribute combinations ordering should be
+    /// sorted using case insensitive comparison
     pub fn write_aggregates_to_string(
         &self,
         aggregates_delimiter: char,
         combination_delimiter: &str,
+        case_insensitive_combinations_order: Option<bool>,
     ) -> Result<String, Error> {
         let mut csv_aggregates = Vec::default();
 
@@ -875,6 +915,7 @@ impl AggregatedData {
             &mut csv_aggregates,
             aggregates_delimiter,
             combination_delimiter,
+            case_insensitive_combinations_order,
         )?;
 
         Ok(String::from_utf8_lossy(&csv_aggregates).to_string())
